(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['module', 'exports', '../utils'], factory);
  } else if (typeof exports !== "undefined") {
    factory(module, exports, require('../utils'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod, mod.exports, global.utils);
    global.index = mod.exports;
  }
})(this, function (module, exports, _utils) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });


  var diff = function diff(lhs, rhs) {
    if (lhs === rhs) return {}; // equal return no diff

    if (!(0, _utils.isObject)(lhs) || !(0, _utils.isObject)(rhs)) return rhs; // return updated rhs

    var l = (0, _utils.properObject)(lhs);
    var r = (0, _utils.properObject)(rhs);

    var changes = {};

    var lKeys = Object.keys(l);

    for (var i = 0; i < lKeys.length; i++) {
      var key = lKeys[i];

      if (!r.hasOwnProperty(key)) {
        changes[key] = undefined;
      }
    }

    if ((0, _utils.isDate)(l) || (0, _utils.isDate)(r)) {
      if (l.valueOf() == r.valueOf()) return {};
      return r;
    }

    if (Array.isArray(r) && Array.isArray(l)) {
      var deletedValues = l.reduce(function (acc, item, index) {
        return r.hasOwnProperty(index) ? acc.concat(item) : acc.concat(undefined);
      }, []);

      return r.reduce(function (acc, rightItem, index) {
        if (!deletedValues.hasOwnProperty(index)) {
          return acc.concat(rightItem);
        }

        var leftItem = l[index];
        var difference = diff(rightItem, leftItem);

        if ((0, _utils.isObject)(difference) && (0, _utils.isEmpty)(difference) && !(0, _utils.isDate)(difference)) {
          delete acc[index];
          return acc; // return no diff
        }

        return acc.slice(0, index).concat(rightItem).concat(acc.slice(index + 1)); // return updated key
      }, deletedValues);
    }

    var rKeys = Object.keys(r);

    for (var _i = 0; _i < rKeys.length; _i++) {
      var _key = rKeys[_i];

      if (!l.hasOwnProperty(_key)) {
        changes[_key] = r[_key];
        continue;
      }

      var difference = diff(l[_key], r[_key]);

      if ((0, _utils.isObject)(difference) && (0, _utils.isEmpty)(difference) && !(0, _utils.isDate)(difference)) continue; // return no diff

      changes[_key] = difference;
    }

    return changes;
  };

  exports.default = diff;
  module.exports = exports['default'];
});